package com.ningdatech.pmapi.projectlib.manage;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.basic.model.PageVo;
import com.ningdatech.basic.util.CollUtils;
import com.ningdatech.pmapi.projectlib.enumeration.ProjectRenewalApprovalStatusEnum;
import com.ningdatech.pmapi.projectlib.enumeration.ProjectStatusEnum;
import com.ningdatech.pmapi.projectlib.model.dto.ProjectRenewalFundDeclarationDTO;
import com.ningdatech.pmapi.projectlib.model.entity.Project;
import com.ningdatech.pmapi.projectlib.model.entity.ProjectRenewalFundDeclaration;
import com.ningdatech.pmapi.projectlib.model.req.ProjectRenewalListReq;
import com.ningdatech.pmapi.projectlib.model.vo.ProjectRenewalFundDeclarationVO;
import com.ningdatech.pmapi.projectlib.service.IProjectRenewalFundDeclarationService;
import com.ningdatech.pmapi.projectlib.service.IProjectService;
import com.wflow.exception.BusinessException;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * ProjectRenewalFundManage
 * </p>
 *
 * @author ZPF
 * @since 14:19 2023/2/15
 */
@Component
@RequiredArgsConstructor
public class ProjectRenewalFundManage {

    private final IProjectRenewalFundDeclarationService projectRenewalFundDeclarationService;

    private final IProjectService projectService;

    /**
     * 项目续建资金库列表 分页
     * @param req
     * @return
     */
    public PageVo<ProjectRenewalFundDeclarationVO> list(ProjectRenewalListReq req) {
        Page<ProjectRenewalFundDeclaration> page = req.page();
        projectRenewalFundDeclarationService.pageSql(page, req);
        long total;
        if ((total = page.getTotal()) == 0) {
            return PageVo.empty();
        }
        List<ProjectRenewalFundDeclarationVO> records = CollUtils.convert(page.getRecords(), w -> {
            ProjectRenewalFundDeclarationVO item = new ProjectRenewalFundDeclarationVO();
            BeanUtils.copyProperties(w,item);
            return item;
        });
        return PageVo.of(records, total);
    }

    /**
     * @param id
     * @author ZPF
     * @since 2023/02/15 11:15
     */
    public ProjectRenewalFundDeclarationVO detail(Long id) {
        // 查询项目申报信息
        ProjectRenewalFundDeclaration renewal = projectRenewalFundDeclarationService.getById(id);
        if(Objects.isNull(renewal)){
            return null;
        }
        ProjectRenewalFundDeclarationVO vo = new ProjectRenewalFundDeclarationVO();
        BeanUtils.copyProperties(renewal, vo);
        // 查询项目信息
        Project project = projectService.getById(renewal.getProjectId());
        if(Objects.isNull(project)){
            return vo;
        }
        vo.setApprovalAmount(project.getApprovalAmount());
        vo.setProjectName(project.getProjectName());
        vo.setProjectType(project.getProjectType());
        vo.setStage(project.getStage());
        vo.setStatus(project.getStatus());
        return vo;
    }

    /**
     * 申报
     * @param dto
     * @return
     */
    public Long declared(ProjectRenewalFundDeclarationDTO dto) {
        //要判断 项目id 是否 以及状态是 已验收
        Long projectId = dto.getProjectId();
        Project project = projectService.getById(projectId);
        VUtils.isTrue(Objects.isNull(project)).throwMessage(String.format("关联的项目【%s】不存在 提交失败!",projectId));
        VUtils.isTrue(!ProjectStatusEnum.ACCEPTED.getCode().equals(project.getStatus()) ||
                        !ProjectStatusEnum.ARCHIVED.getCode().equals(project.getStage()))
                .throwMessage("提交失败 该项目不是 已验收状态或者已归档阶段");
        ProjectRenewalFundDeclaration declaration = new ProjectRenewalFundDeclaration();
        //如果是重新申报
        if(Objects.nonNull(dto.getId())){
            ProjectRenewalFundDeclaration oldDeclaration = projectRenewalFundDeclarationService.getById(dto.getId());
            VUtils.isTrue(Objects.isNull(oldDeclaration)).throwMessage(String.format("重新申报失败 【%s】 该申报不存在!",dto.getId()));
            BeanUtils.copyProperties(oldDeclaration,declaration);
        }
        BeanUtils.copyProperties(dto,declaration);
        declaration.setCreateOn(LocalDateTime.now());
        declaration.setUpdateOn(LocalDateTime.now());
        declaration.setApprovalStatus(ProjectRenewalApprovalStatusEnum.PENDING.name());
        if(!projectRenewalFundDeclarationService.saveOrUpdate(declaration)){
            throw new BusinessException(String.format("申报失败 【%s】",dto.getId()));
        }
        return declaration.getId();
    }
}
